# -*- coding: utf-8 -*-
from future.utils import python_2_unicode_compatible


@python_2_unicode_compatible
class Model(object):
    """
    base class from which all Nation Address models will initiate
    """

    def __init__(self, **kwargs):
        self.param_defaults = {}

    @classmethod
    def from_json(cls, data, **kwargs):
        """
        Create a new instance from a JSON dict
        """
        data_copy = data.copy()
        if kwargs:
            for key, value in kwargs.items():
                data_copy[key] = value

        c = cls(**data_copy)
        c._json = data
        return c


class Address(Model):
    """
    A class representation of the Address component
    """

    def __init__(self, **kwargs):
        self.param_defaults = {
            'Title': None,
            'Address1': None,
            'Address2': None,
            'ObjLatLng': None,
            'BuildingNumber': None,
            'Street': None,
            'District': None,
            'City': None,
            'PostCode': None,
            'AdditionalNumber': None,
            'UnitNumber': None,
            'RegionName': None,
            'IsPrimaryAddress': None,
            'Latitude': None,
            'Longitude': None,
        }
        for param, default in self.param_defaults.items():
            value = kwargs.get(param, default)
            value = value if value else ''
            setattr(self, param, value)

    def __repr__(self):
        return u"{building} {street} - {dist} {city} {post_code} - {add_no}".format(building=self.BuildingNumber,
                                                                                    street=self.Street,
                                                                                    dist=self.District, city=self.City,
                                                                                    post_code=self.PostCode,
                                                                                    add_no=self.AdditionalNumber)


class Region(Model):
    """
    A class representation of the Region component
    """

    def __init__(self, **kwargs):
        self.param_defaults = {
            'Id': None,
            'Name': None,
        }
        for param, default in self.param_defaults.items():
            value = kwargs.get(param, default)
            value = value if value else ''
            setattr(self, param, value)

    def __repr__(self):
        return u"Region (ID={id}, Name={name})".format(id=self.Id, name=self.Name)


class City(Model):
    """
    A class representation of the City component
    """

    def __init__(self, **kwargs):
        self.param_defaults = {
            'Id': None,
            'Name': None,
        }
        for param, default in self.param_defaults.items():
            value = kwargs.get(param, default)
            value = value if value else ''
            setattr(self, param, value)

    def __repr__(self):
        return u"City (ID={id}, Name={name})".format(id=self.Id, name=self.Name)


class District(Model):
    """
    A class representation of the District component
    """

    def __init__(self, **kwargs):
        self.param_defaults = {
            'Id': None,
            'Name': None,
        }
        for param, default in self.param_defaults.items():
            value = kwargs.get(param, default)
            value = value if value else ''
            setattr(self, param, value)

    def __repr__(self):
        return u"District (ID={id}, Name={name})".format(id=self.Id, name=self.Name)


class ServiceCategory(Model):
    """
    A class representation of the Service Category and sub-Category component
    """

    def __init__(self, **kwargs):
        self.param_defaults = {
            'Id': None,
            'Name': None,
        }
        for param, default in self.param_defaults.items():
            value = kwargs.get(param, default)
            value = value if value else ''
            setattr(self, param, value)

    def __repr__(self):
        return u"Category (ID={id}, Name={name})".format(id=self.Id, name=self.Name)


class ServiceSubCategory(Model):
    """
    A class representation of the Service Category and sub-Category component
    """

    def __init__(self, **kwargs):
        self.param_defaults = {
            'Id': None,
            'Name': None,
        }
        for param, default in self.param_defaults.items():
            value = kwargs.get(param, default)
            value = value if value else ''
            setattr(self, param, value)

    def __repr__(self):
        return u"Sub-Category (ID={id}, Name={name})".format(id=self.Id, name=self.Name)


class ResultSet(object):
    """
    A holder class for base model instances
    """

    def __init__(self, records, count):
        self.records = records
        self.count = count



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function
import requests
from .models import ResultSet, Address, Region, City, District, ServiceCategory, ServiceSubCategory

SA_NATIONAL_ADDRESS_URL = 'https://apina.address.gov.sa'
FORMAT = 'json'
ENCODE = 'utf8'


class NationalAddress(object):
    def __init__(self, url=None, key=None):
        if url:
            self.url = url
        else:
            self.url = SA_NATIONAL_ADDRESS_URL

        if key:
            self.key = key
        else:
            raise ValueError('no api_key was provided')

    def get_address(self, lat, long, language='E'):
        """Address reverse Geocode
        Args:
            lat: latitude in decimal unit
            long: longitude in decimal unit
            language: A for Arabic, E for English
        """
        headers = {'api_key': self.key}
        params = {'lat': lat, 'long': long, 'language': language, 'format': FORMAT, 'encode': ENCODE}
        r = requests.get(self.url + '/NationalAddress/v3.1/Address/address-geocode', params=params, headers=headers)
        result = r.json()
        addresses = []
        total = 0
        if result.get('success'):
            records = result.get('Addresses')
            total = result.get('totalSearchResults', 0)
            if records:
                for record in records:
                    addresses.append(Address.from_json(record))

        rs = ResultSet(addresses, int(total))
        return rs

    def bulk_search(self, addresses_tuple, language='E', page=1):
        """Search bulk addresses for complete details
        Args:
            addresses_tuple: a list of tuple in the form [(building_number, zip_code, additional_number)]
            language: A for Arabic, E for English
            page: if search count > 10, then this must be passed along to get the next set of results.
        """
        addresses_list = []
        for address in addresses_tuple:
            addresses_list.append('|'.join(str(attribute) for attribute in address))
        address_string = ';'.join(addresses_list)
        headers = {'api_key': self.key}
        params = {'addressstring': address_string, 'language': language, 'format': FORMAT, 'encode': ENCODE,
                  'page': page}
        r = requests.get(self.url + '/NationalAddress/v3.1/Address/address-bulk', params=params, headers=headers)
        result = r.json()
        addresses = []
        total = 0
        if result.get('success'):
            records = result.get('Addresses')
            total = result.get('totalSearchResults', 0)
            if records:
                for record in records:
                    addresses.append(Address.from_json(record))

        rs = ResultSet(addresses, int(total))
        return rs

    def fixed_search(self, language='E', page=1, city_id=None, district_id=None, building_number=None, post_code=None,
                     additional_number=None, city_name=None, district_name=None, street_name=None):
        """Find a complete address by providing some details about it"""
        headers = {'api_key': self.key}
        params = {'language': language, 'page': page, 'format': FORMAT, 'encode': ENCODE, 'cityId': city_id,
                  'districtId': district_id, 'buildingnumber': building_number, 'zipcode': post_code,
                  'additionalnumber': additional_number, 'cityname': city_name, 'districtname': district_name,
                  'streetname': street_name}
        r = requests.get(self.url + '/NationalAddress/v3.1/Address/address-fixed-params', params=params,
                         headers=headers)
        result = r.json()
        addresses = []
        total = 0
        if result.get('success'):
            records = result.get('Addresses')
            total = result.get('totalSearchResults', 0)
            if records:
                for record in records:
                    addresses.append(Address.from_json(record))

        rs = ResultSet(addresses, int(total))
        return rs

    def free_text_search(self, address_string, language='E', page=1):
        """Find a complete address by providing some details in free text"""
        headers = {'api_key': self.key}
        params = {'addressstring': address_string, 'language': language, 'format': FORMAT, 'encode': ENCODE,
                  'page': page}
        r = requests.get(self.url + '/NationalAddress/v3.1/Address/address-free-text', params=params, headers=headers)
        result = r.json()
        addresses = []
        total = 0
        if result.get('success'):
            records = result.get('Addresses')
            total = result.get('totalSearchResults', 0)
            if records:
                for record in records:
                    addresses.append(Address.from_json(record))

        rs = ResultSet(addresses, int(total))
        return rs

    def nearest_poi(self, lat, long, language='E', page=1, radius=0.5):
        """Search nearest service"""
        headers = {'api_key': self.key}
        params = {'lat': lat, 'long': long, 'language': language, 'format': FORMAT, 'encode': ENCODE,
                  'page': page, 'radius': radius}
        r = requests.get(self.url + '/NationalAddress/v3.1/Address/poi-nearest', params=params, headers=headers)
        result = r.json()
        addresses = []
        total = 0
        if result.get('success'):
            records = result.get('Addresses')
            total = result.get('totalSearchResults', 0)
            if records:
                for record in records:
                    addresses.append(Address.from_json(record))

        rs = ResultSet(addresses, int(total))
        return rs

    def poi_fixed_search(self, service_string, language='E', page=1, city_id=None, district_id=None,
                         building_number=None, post_code=None, additional_number=None, city_name=None,
                         district_name=None, street_name=None, service_category_id=None, service_subcategory_id=None,
                         region_id=None):
        """Find availability of a service by providing some details in fixed parameters"""
        headers = {'api_key': self.key}
        params = {'servicestring': service_string, 'language': language, 'format': FORMAT, 'encode': ENCODE,
                  'page': page, 'cityId': city_id, 'districtId': district_id, 'buildingnumber': building_number,
                  'zipcode': post_code, 'additionalnumber': additional_number, 'cityname': city_name,
                  'districtname': district_name, 'streetname': street_name, 'servicecategoryId': service_category_id,
                  'servicesubcategoryId': service_subcategory_id, 'regionid': region_id}
        r = requests.get(self.url + '/NationalAddress/v3.1/Address/poi-fixed-params', params=params, headers=headers)
        result = r.json()
        addresses = []
        total = 0
        if result.get('success'):
            records = result.get('Addresses')
            total = result.get('totalSearchResults', 0)
            if records:
                for record in records:
                    addresses.append(Address.from_json(record))

        rs = ResultSet(addresses, int(total))
        return rs

    def poi_free_text_search(self, service_string, language='E', page=1):
        """Find availability of a service by providing some details in free text"""
        headers = {'api_key': self.key}
        params = {'servicestring': service_string, 'language': language, 'format': FORMAT, 'encode': ENCODE,
                  'page': page}
        r = requests.get(self.url + '/NationalAddress/v3.1/Address/poi-free-text', params=params, headers=headers)
        result = r.json()
        addresses = []
        total = 0
        if result.get('success'):
            records = result.get('Addresses')
            total = result.get('totalSearchResults', 0)
            if records:
                for record in records:
                    addresses.append(Address.from_json(record))

        rs = ResultSet(addresses, int(total))
        return rs

    def verify_address(self, building_number, post_code, additional_number, language='E', page=1):
        """Verify an address"""
        headers = {'api_key': self.key}
        params = {'buildingnumber': building_number, 'zipcode': post_code, 'additionalnumber': additional_number,
                  'language': language, 'format': FORMAT, 'encode': ENCODE, 'page': page}
        r = requests.get(self.url + '/NationalAddress/v3.1/Address/address-verify', params=params, headers=headers)
        result = r.json()
        if result.get('success'):
            return result.get('addressfound')

    def get_regions(self, language='E'):
        """Get a lookup list of regions"""
        headers = {'api_key': self.key}
        params = {'language': language, 'format': FORMAT, 'encode': ENCODE}
        r = requests.get(self.url + '/NationalAddress/v3.1/lookup/regions', params=params, headers=headers)
        result = r.json()
        lookup = []
        if result.get('success'):
            records = result.get('Regions')
            if records:
                for record in records:
                    lookup.append(Region.from_json(record))

        return lookup

    def get_cities(self, region_id=-1, language='E'):
        """Get a lookup list of cities"""
        headers = {'api_key': self.key}
        params = {'language': language, 'format': FORMAT, 'regionid': region_id}
        r = requests.get(self.url + '/NationalAddress/v3.1/lookup/cities', params=params, headers=headers)
        result = r.json()
        lookup = []
        if result.get('success'):
            records = result.get('Cities')
            if records:
                for record in records:
                    lookup.append(City.from_json(record))

        return lookup

    def get_districts(self, city_id=-1, language='E'):
        """Get a lookup list of districts"""
        headers = {'api_key': self.key}
        params = {'language': language, 'format': FORMAT, 'cityid': city_id}
        r = requests.get(self.url + '/NationalAddress/v3.1/lookup/districts', params=params, headers=headers)
        result = r.json()
        lookup = []
        if result.get('success'):
            records = result.get('Districts')
            if records:
                for record in records:
                    lookup.append(District.from_json(record))

        return lookup

    def get_categories(self, language='E'):
        """Get a lookup list of service categories"""
        headers = {'api_key': self.key}
        params = {'language': language, 'format': FORMAT}
        r = requests.get(self.url + '/NationalAddress/v3.1/lookup/service-categories', params=params, headers=headers)
        result = r.json()
        lookup = []
        if result.get('success'):
            records = result.get('ServiceCategories')
            if records:
                for record in records:
                    lookup.append(ServiceCategory.from_json(record))

        return lookup

    def get_subcategories(self, service_category_id=-1, language='E'):
        """Get a lookup list of service sub-categories"""
        headers = {'api_key': self.key}
        params = {'language': language, 'format': FORMAT, 'servicecategoryid': service_category_id}
        r = requests.get(self.url + '/NationalAddress/v3.1/lookup/services-sub-categories', params=params,
                         headers=headers)
        result = r.json()
        lookup = []
        if result.get('success'):
            records = result.get('ServiceSubCategories')
            if records:
                for record in records:
                    lookup.append(ServiceSubCategory.from_json(record))

        return lookup



# coding: utf-8

# In[2]:


from __future__ import print_function
from __future__ import unicode_literals
import os
import time
from saudiaddress.api import NationalAddress


# In[3]:

# initialize
DELAY_IN_SEC = 5

api_key = os.environ.get('NATIONAL_ADDRESS_API_KEY')

na = NationalAddress(key=api_key)


# In[4]:

# to get a national address from lat/long 
addresses = na.get_address(lat=26.318922, long=50.228043, language='E')
print("address:", addresses.records[0])


# In[5]:

time.sleep(DELAY_IN_SEC)
# to bulk search a list of (building_no, post_code, additional_no)
for address in na.bulk_search([(3468, 23955, 7487), (6418, 34412, 3618)], page=1).records:
    print(address)


# In[6]:

time.sleep(DELAY_IN_SEC)
# to verify an address
print("the address is", na.verify_address(6418, 34412, 3618))


# In[7]:

time.sleep(DELAY_IN_SEC)
# free text search for an address
addresses = na.free_text_search('2292  - king abdullah university of science and technology')
print("count:", addresses.count)
address = addresses.records[0]
print('lat={lat}, long={long}, city={city}, region={region}'.format(lat=address.Latitude, long=address.Longitude,
                                                                    city=address.City, region=address.RegionName))


# In[8]:

time.sleep(DELAY_IN_SEC)
# search nearest services
addresses = na.nearest_poi(lat=22.32, long=39.09, radius=2)
for address in addresses.records:
    print('title:%s' % address.Title)
    print('address:', address)


# In[9]:

time.sleep(DELAY_IN_SEC)
# search services based on fixed parameters
addresses = na.fixed_search(city_name='THUWAL', post_code=23955, additional_number=7487)
print(addresses.records[0])


# In[11]:

time.sleep(DELAY_IN_SEC)
# find availability of services by providing details in free text
addresses = na.poi_free_text_search('sabb atm', language='E')
print('total SABB ATMs found: %d' % addresses.count)
print(addresses.records)


# In[4]:

time.sleep(DELAY_IN_SEC)
# get a list of regions
regions = na.get_regions(language='A')
for region in regions[:5]:
    print(region)


# In[6]:

time.sleep(DELAY_IN_SEC)
# get a list of cities in a region
cities = na.get_cities(region_id=1)
for city in cities[:5]:
    print(city)


# In[14]:

time.sleep(DELAY_IN_SEC)
# get a list of districts in a city
districts = na.get_districts(city_id=3)
for district in districts[:5]:
    print(district)


# In[9]:

time.sleep(DELAY_IN_SEC)
# get a list of service categories
categories = na.get_categories()
for category in categories[:5]:
    print(category)


# In[13]:

time.sleep(DELAY_IN_SEC)
# to get a list of sub-service categories
sub_categories = na.get_subcategories(service_category_id=101)
for sub_category in sub_categories[:5]:
    print(sub_category)


# In[20]:

time.sleep(DELAY_IN_SEC)
# find all ATMs in district ad dirah in AR RIYADH city
services = na.poi_fixed_search('atm', district_name='ad dirah', city_name='AR RIYADH', page=1)
print("total:", services.count)
for service in services.records:
    print(service.Title, service)


# In[ ]:






#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
import sys
from shutil import rmtree

from setuptools import setup, Command, find_packages

NAME = 'saudiaddress'
DESCRIPTION = 'A simple Python wrapper for the Saudi National Address APIs.'
URL = 'https://github.com/abduhbm/saudiaddress'
AUTHOR = 'Abdulelah Bin Mahfoodh'

REQUIRED = [
    'requests',
]

here = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = '\n' + f.read()

about = {}
with open(os.path.join(here, NAME, '__version__.py')) as f:
    exec(f.read(), about)


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        sys.exit()


setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    author=AUTHOR,
    # author_email=EMAIL,
    url=URL,
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    install_requires=REQUIRED,
    license='MIT',
    keywords='Saudi National Address',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Topic :: Utilities'
    ],
    # $ setup.py publish support.
    cmdclass={
        'upload': UploadCommand,
    },
)